###############################################################
#   
#  File:      parameter_spec.rb
#
#  Author:    Christian Hättich
#
#  Project:   System-On-Chip Maker
#
#  Target:    Linux / Windows / Mac
#
#  Language:  ruby
#
#
###############################################################
#
#
#   Copyright (C) 2014  Christian Hättich  - feddischson [ at ] opencores.org
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
###############################################################
#
#   Description:
#     Test specification for SOCMaker::Parameter
#
#
#
#
###############################################################
require_relative( 'spec_helper' )

require "yaml_examples.rb"


describe SOCMaker::Parameter do

  describe "#initialize" do

    it "returns an object of type SOCMaker::Parameter when creating it with new" do
      o = SOCMaker::Parameter.new( "integer" )
      expect( o.class).to be( SOCMaker::Parameter )
    end

    it "raises an error if the type is not a string" do
      expect{ SOCMaker::Parameter.new( 3 ) }.
      to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the type is a string with zero length" do
      expect{ SOCMaker::Parameter.new( "" ) }.
      to raise_error( SOCMaker::ERR::InitError )
    end

    it "raises an error if the type is nil" do
      expect{ SOCMaker::Parameter.new( "" ) }.
      to raise_error( SOCMaker::ERR::InitError )
    end
  end

  describe "SOCMaker::from_s" do
    include_context "yaml examples"

    it 'raises an error if the type value of an instance param. is empty' do
      expect{ SOCMaker::from_s( @F_YAML_INSTP_TYPE ) }.
      to raise_error( SOCMaker::ERR::InitError )
    end
  end
end


# vim: noai:ts=2:sw=2



