


RSpec.shared_context "yaml examples", :a => :b  do

 before do

  @valid_yamls = []


  #
  # A yaml example, which contains 
  #   - a full definition
  #   - an interface with one port
  #   - one hdl file
  #   - one instance parameter
  #   - one static parameter
  # 
  @FULL_YAML = '''SOCM_CORE
name: core_A
description: A test IP-core
id: core_A,rel1
date: 1.1.2014
license: LGPL
licensefile: 
author: Christian Haettich
authormail: feddischson@opencores.org
vccmd: svn co http://some-address/
toplevel: core_a
interfaces:
  :ifc01: SOCM_IFC
    name: core_AB_ifc
    dir: 0
    id: core_AB_ifc,1
    ports:
      :sig_con1a: SOCM_PORT
         spc_ref: sig_a
         len:  param1
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      use_syn: true
      use_sys_sim: true
      use_mod_sim: true
      type: vhdl
      path: ./core_a.vhd
inst_parameters:
  :param1: SOCM_PARAM
    type: integer
    default: 8
    min: 0
    max: 10
    visible: true
    editable: true
    description: More setup
static_parameters:
  :core_a_pkg.vhd.src: SOCM_SPARAM
    path: ./core_a.pkg.src
    file_dst: core_a_pkg.vhd
    parameters: 
      :p1: SOCM_SENTRY
        token: TOK1
        type: integer
        min:  0
        max:  100
        visible: true
        editable: true
        default: 3
        description: Some setup
'''
  @valid_yamls << {  desc: 'should return a class of type CoreDef when loading a full core-def',
                    yaml: @FULL_YAML }



  # minimalistic def with one vhdl file
  @MIN_YAML1 = '''SOCM_CORE
name: core_A
id: core_A,rel1
toplevel: top_A
hdlfiles: 
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
'''
  @valid_yamls << {  desc: 'should return a class of type CoreDef when loading a minimal core-def',
                    yaml: @MIN_YAML1 }


  # minimalistic def with one 
  # vhdl and one verilog file
  @MIN_YAML2 = '''SOCM_CORE
name: core_A
id: core_A,rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
   :core_b.v: SOCM_HDL_FILE
      path: ./core_b.v
'''
  @valid_yamls << {  desc: 'should return a class of type CoreDef when loading a core-def with two files',
                    yaml: @MIN_YAML2 }



  # minimal setup with one interface
  @MIN_YAML_IFC = '''SOCM_CORE
name: core_A
id: core_A,rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
interfaces:
  :ifc01: SOCM_IFC
    name: a_ifc_def
    dir: 0
    id: a_ifc_def,1
    ports:
      :sig_con1a: SOCM_PORT
         spc_ref: sig_a
'''
  @valid_yamls << { desc: 'should return a class of type CoreDef when loading a minimal core-def with a minimal interface',
                   yaml: @MIN_YAML_IFC }





  # minimal setup with one instance parameter
  @MIN_YAML_INSTP = '''SOCM_CORE
name: core_A
id: core_A,rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
inst_parameters: 
  :param1: SOCM_PARAM
    type: integer
'''
  @valid_yamls << {  desc: 'should return a class of type CoreDef when loading a minimal core-def (with instance param.)',
                    yaml: @MIN_YAML_INSTP }



  # minimal def with one static parameter
  @MIN_YAML_STATIC = '''SOCM_CORE
name: core_A
id: core_A,rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
static_parameters:
  :a_file.vhd.src: SOCM_SPARAM
    file_dst: a_file.vhd
    path: ./core_a.vhd
    parameters:
      :p1:  SOCM_SENTRY
        type: integer
        token: T1
'''
  @valid_yamls << {  desc: 'should return a class of type CoreDef when loading a minimal core-def (with static param.)', 
                    yaml: @MIN_YAML_STATIC }





  #
  # A yaml example, which contains 
  #   - a full definition
  #   - an interface with one port
  #   - one hdl file
  #   - one instance parameter
  #   - one static parameter
  # 
  @FULL_SOC_YAML = '''SOCM_SOC
name: my_soc
description: A test SOC
id: my_soc,rel2
date: 1.1.2014
license: LGPL
licensefile: 
toplevel: soc_top
author: Christian Haettich
authormail: feddischson@opencores.org
repocmd: svn co http://some-address/
'''


  @SOC_YAML_WITH_CORE = '''SOCM_SOC
name: my_soc
description: A test SOC
id: my_soc,rel2
date: 1.1.2014
license: LGPL
licensefile: 
author: Christian Haettich
authormail: feddischson@opencores.org
repocmd: svn co http://some-address/
toplevel: soc_top
cores:
  :inst1: SOCM_INST
    type: mycorerel1
'''


  @F_YAML_STATIC_NO_TOKEN = '''SOCM_CORE
name: core_A
version: rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
static_parameters: 
  :a_file.vhd.src: SOCM_SPARAM
    path: ./core_a.vhd
    file_dst: a_file.vhd
    parameters:
      :p1:  SOCM_SENTRY
        min: 4
        type: integer
'''

  @F_YAML_STATIC_NO_PATH = '''SOCM_CORE
name: core_A
version: rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
static_parameters: 
  :a_file.vhd.src: SOCM_SPARAM
    file_dst: a_file.vhd
    parameters:
      :p1:  SOCM_SENTRY
        min: 4
        token: ABC
        type: integer
'''

  @F_YAML_STATIC_NO_TYPE = '''SOCM_CORE
name: core_A
version: rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
static_parameters: 
  :a_file.vhd.src: SOCM_SPARAM
    file_dst: a_file.vhd
    path: ./core_a.vhd
    parameters:
      :p1:  SOCM_SENTRY
        min: 4
        token: ABC
'''

  # missing file_dst in static parameter
  @F_YAML_STATIC_NO_DST1 = '''SOCM_CORE
name: core_A
version: rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
static_parameters:
  :a_file.vhd: SOCM_SPARAM
    parameters:
      :p1: SOCM_SENTRY
        type: integer
        token: T1
'''
  # no parameter entry
  @F_YAML_STATIC_NO_SENTRY = '''SOCM_CORE
name: core_A
version: rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
static_parameters:
  :a_file.vhd.src: SOCM_SPARAM
    file_dst: a_file_xyz.vhd
    path: ./core_a.vhd
    parameters:
      :p1: SOCM_SENTRY
'''

  # type is missing for param1
  @F_YAML_INSTP_TYPE = '''SOCM_CORE
name: core_A
version: rel1
toplevel: top_A
hdlfiles:
   :core_a.vhd: SOCM_HDL_FILE
      path: ./core_a.vhd
inst_parameters:
  :param1: SOCM_PARAM
    :min: 0
'''


  @IFC_YAML_VALID = """
SOCM_IFC_SPC
name: core_ifc
id: 'core_ifc,1'
ports:
  :sig_a: 
    :dir: 1
  :sig_b: 
    :dir: 1
  :sig_c: 
    :dir: 0
"""

  @IFC_YAML_INVALID = """
SOCM_IFC_SPC
name: core_ifc
id: 'core_ifc,1'
ports:
  :sig_a:
    :dir: '12'
  :sig_b: 
    :dir: 1
  :sig_c: 
    :dir: 0
"""

  @IFC_YAML_INVALID2 = """
SOCM_IFC_SPC
name: core_ifc
id: 'core_ifc,1'
ports:
  :sig_a:
    :dir:
  :sig_b: 
    :dir: 1
  :sig_c: 
    :dir: 0
"""

  #
  # core definition with no info about the HDL file
  @F_MIN_YAML_FIlE_NIL = '''SOCM_CORE
  name: core_A
  version: rel1
  toplevel: top_A
  hdlfiles:
     :a_file.v: SOCM_HDL_FILE
  '''

  @F_YAML_FILE_TYPE = '''SOCM_CORE
  name: core_A
  version: rel1
  toplevel: top_A
  hdlfiles:
     :core_a.vhd: SOCM_HDL_FILE
        path: ./core_a.vhd
        type: xhdl
  '''


  end

end
